#!/usr/bin/python
# -*- coding: utf-8 -*-
#

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Datos: Todas las clases relacionadas con los datos, 
y sus correspondientes metodos
"""
from Driza import variables 
import types
from qt import QString #TODO: Poner solo en el lugar que sea necesario


class Registro(list):
    """Almacena un registro, permitiendo el acceso tanto por el indice, 
    como por el nombre de variable.  Esta clase garantiza que la lista de 
    valores se ajusta a los tipos de la lista de variables
    Hereda de list
    """

    def __init__(self, lvariables):
        """ 
        Clase constructora
        lvariables es el array de variables al que esta asociado el registro
        Añade los valores por defecto para todos los atributos
        """
        #VARIABLES PRIVADAS
        self.__variables = lvariables

        list.__init__(self)
        for var in self.__variables.arr_defaults():
            self.append(var)    
    
    #FUNCIONES PUBLICAS
    def __getitem__(self, indice):
        """ Redefinicion del [] de la lista. 
        Si nos pasan una ristra, comprobamos si corresponde a alguna variable
        Si nos pasan un objeto, comprobamos si se corresponde a alguna variable
        Si nos pasan un entero, devolvemos el valor de esa posicion
        """
        miindice = self.__variables.obtenerindice(indice)
        return list.__getitem__(self, miindice)

    def __setitem__(self, indice, valor):
        """ 
        Redefinicion de []= de la lista. Almacena en el valor 
        convertido a formato interno por la variable asociada
        """
        miindice = self.__variables.obtenerindice(indice)
        if not isinstance(valor, str):
            valor = repr(valor)
        list.__setitem__(self, miindice, self.__variables[miindice].nuevoitem\
                (valor))

    def append(self, valor):
        """ 
        Redefinicion del append de lista. Añade al registro el valor pasado 
        en el formato interno de la variable asociada 
        """
        #TODO: Este append es problemático, puesto que depende de que 
        #la variable ya exista y ademas sea la última. Pendiente de rediseñar
        intermedio = self.__variables[self.__len__()].nuevoitem(valor)
        list.append(self, intermedio)
        
    def set(self, lista):
        """ 
        Esta funcion establece los valores del registro, 
        teniendo como argumento una lista o derivado 
        """
        assert(isinstance(lista,list))
        i = 0
        for valor in lista:
            self.__setitem__(i, valor)
            i += 1


class ListaVar(list): 
    """
    ListaVar hereda de list. Su utilidad es almacenar un conjunto de 
    variables, permitiendo recuperarlas no solo por el indice, sino por 
    el nombre.
    """

    def __getcopy__(self):
        resultado = ListaVar()
        for i in self:
            resultado.append(i)
        return resultado

    #FUNCIONES PUBLICAS

    def arr_defaults(self):
        """
        Devuelve una lista con las ristras de todos 
        los valores por defecto
        """
        return [x.valorpordefecto for x in self]

    def obtenerindice(self, parametro):
        """
         Permite obtener el indice de una variable a partir de su nombre, 
         del objeto, o de un entero.  Devuelve un entero.
        """
        if isinstance(parametro, QString):
            parametro = parametro.latin1()

        if parametro.__class__ == types.StringType:
            for iter in self:  
                if iter.nombre == parametro:
                    return self.index(iter)
            return False
        elif parametro.__class__ == types.IntType:
            return parametro
        else: #El parametro lo tomamos como una referencia
            if parametro in self:
                return self.index(parametro)
            return False



class ContenedorRegVar:
    """
    Esta clase almacena los datos del programa. Tiene dos miembros, 
    una lista de registros y una lista de variables
    """
    def __init__(self, registros = None, variables = None):
        """Inicialización del contenedor principal"""
        if not registros: 
            registros=[]
        self.__reg = registros #Un array para datos
        if not variables: 
            variables = ListaVar()
        self.__var = variables #Un array de variables 

    def __deepcopy__(self, memo):
        """Reimplementación de __deepcopy__, permite una copia del objeto"""
        nuevalista = self.__var.__getcopy__()
        nuevalistaregistros = list()
        for registro in self.__reg:
            nuevoregistro = Registro(nuevalista)
            for x in range(len(registro)):
                nuevoregistro[x] = nuevalista[x].nuevoitem(repr(registro[x]))
            nuevalistaregistros.append(nuevoregistro)
        resultado= ContenedorRegVar(nuevalistaregistros, nuevalista)
        memo[id(self)] = resultado
        return resultado

    def registros(self):
        return self.__reg
    def variables(self):
        return self.__var




class ListaUndoRedo(list):
    """ListaUndoRedo hereda de lista. Es similar a una cola lifo.
    Los elementos almacenados son tuplas (estado,flagoriginal)
    """
    def __init__(self, tama):
        self.tama = tama #Almacena el tamaño almacenado

    def encolar(self, objeto, flagoriginal = False, index = 0):
        """Si pasan un indice es que quieren encolar desde alguna posición"""
        if index>=self.tama:
            raise IndexError
        else:
            if self.__len__() == self.tama:
                self.__delitem__(-1)
            self.__delslice__(0, index)
            self.insert(0, (objeto,flagoriginal))

    def setFlag(self, indice,valor):
        """Modifica la flag de original de un indice dado"""
        if (valor!=True and valor!=False):
            raise TypeError
        estadoactual=self.__getitem__(indice)[0]
        self.__setitem__(indice,(estadoactual,valor))

class PorteroDatos:
    """
    Clase de portera. Gestiona una lista con todas las copias de los datos, 
    determinando cual es la vigente y permitiendo el undo y el redo
    """
    def __init__(self,config):
        self.__index=0 #Variable que lleva el indice
        self.__lista = ListaUndoRedo(config.configuracion["nundo"])
        self.__lista.encolar(ContenedorRegVar(), True)

    def undo(self):
        """Realiza un undo, desplazando el indice de ListaUndoRedo"""
        if self.__index < self.__lista.tama:
            self.__index += 1

    def redo(self):
        """Realiza un redo, desplazando el indice de ListaUndoRedo"""
        if self.__index > 0:
            self.__index -= 1

    def puedoundo(self):
        """Devuelve un valor lógico indicando si es posible realizar un undo"""
        return ((self.__index<self.__lista.tama) and (len(self.__lista)>1) and \
                (self.__index<(len(self.__lista)-1)))

    def puedoredo(self):
        """Devuelve un valor lógico indicando si es posible realizar un redo"""
        return self.__index > 0

    def guardarestado(self, flagoriginal = False):
        """
        guarda un estado en la lista de estados, 
        creando una copia a partir del último
        La flagoriginal se activa si se esta abriendo un nuevo fichero
        """
        tmp = self.__lista[self.__index][0]
        import copy
        objeto= copy.deepcopy(tmp)
        self.__lista.encolar(objeto, flagoriginal, self.__index)
    
    def insertarestado(self, estado, flagoriginal = False):
        """Introduce un estado y lo encola"""
        self.__lista.encolar(estado, flagoriginal, self.__index)

    def nuevoestado(self, flagoriginal = False):
        """Introduce un estado nuevo y vacio"""
        self.__lista.encolar(ContenedorRegVar(), flagoriginal)

    def actual(self):
        """Devuelve el estado vigente"""
        return self.__lista[self.__index][0]

    def actualoriginal(self):
        """Devuelve si el estado vigente es original"""
        return self.__lista[self.__index][1]

    def setactualoriginal(self):
        """Establece que el estado actual es original"""
        self.__lista.setFlag(self.__index,True)

class InterfazDatos:
    """
    Interfazdatos pretende ser una interfaz homogenea de acceso y modificacion
    de los datos para grid y otros elementos del programa
    La clase ofrece multiples formas de accesso a los datos internos
    """

    def __init__(self, portero, gestorfiltro, gestorpaquetes):
        """Inicialización del contenedor principal"""
        self._portero = portero
        self.__gestorfiltro = gestorfiltro
        self.__gestorpaquetes = gestorpaquetes

    #FUNCIONES PUBLICAS

    def __getitem__(self, indice): #Acceso por []
        """
        Devuelve el item segun un indice numerico, un nombre, 
        o un objeto variable
        Si el indice es un indice, se entiende que se quiere 
        acceder a un registro
        Si el indice es un nombre o el objeto variable, se entiende 
        que se quiere acceder a la columna
        """
        if indice.__class__ == types.IntType:
            return self._portero.actual().registros()[indice]
        else:
            return self.getCol(indice, filtrado = True)
            
    def __setitem__(self, indice, valor): 
        """
            Modificacion por []
            Si el indice es un entero, se entiende que se 
            quiere modificar a un registro
            Si el indice es un nombre o el objeto variable, 
            se entiende que se quiere modificar a la columna
        """
        if indice.__class__ == types.IntType:
            self._portero.actual().registros[indice]=valor
        else: 
            #Añade los registros que no existen
            for registro in range(len(valor)-len(self._portero.actual().\
                    registros())):
                self.appReg()

            for registro in range(len(self._portero.actual().registros())):
                self._portero.actual().registros()[registro][indice] \
                        = valor[registro]

    def __delitem__(self, indice):
        """define el borrado con la función del, borra un registro"""
        if indice.__class__ == types.IntType:
            self._portero.actual().registros.__delitem__(indice)

    def borrartodo(self):
        """Borra todos los datos"""
        self._portero.nuevoestado(True)

    def getVar(self, indice=-1):
        """Devuelve la variable indicada por el indice, por defecto la última"""
        return self._portero.actual().variables()[self._portero.actual()\
                .variables().obtenerindice(indice)] 
    def numReg(self):
        """Devuelve el número de registros almacenados"""
        return len(self._portero.actual().registros())

    def numVar(self):
        """Devuelve el número de variables almacenadas"""
        return len(self._portero.actual().variables())

    def insReg(self, pos, dato):
        """inserta un registro en la posicion pos"""
        if isinstance(dato, Registro):
            self._portero.actual().registros.insert(pos, dato)
        else:
            registro = Registro(self._portero.actual().variables())
            if dato.__class__ == types.ListType: #Controlar otros casos
                registro.set(dato)
            self._portero.actual().registros().insert(pos, registro)

    def appReg(self,dato=None):
        pos=self.numReg()+1
        self.insReg(pos,dato)

    def appVar(self, nombre = None, tipo = "Entero", valores = {}, \
            valorpordefecto=0, descripcion = "", expresion = None):  
        """
        Añade una variable de tipo tipo al array de variables. 
        Rellena los registros con el valor por defecto, 
        salvo que se indique una expresion
        """
        if not nombre:
            nombre="VAR"+str(self.numVar())
        if nombre in self.arrTit(factores = True):
            from  Driza.excepciones import VariableExisteException
            raise VariableExisteException()
        if tipo == "Entero":
            self._portero.actual().variables().append\
                    (variables.AgrupadorEntero\
                    (nombre, valorpordefecto, descripcion)) 
        elif tipo == "Real":
            self._portero.actual().variables().append\
                    (variables.AgrupadorReal\
                    (nombre, valorpordefecto, descripcion)) 
        elif tipo == "Factor":
            self._portero.actual().variables().append\
                    (variables.AgrupadorFactor(nombre, descripcion)) 
        else:
            raise TypeError #PENDIENTE: Averiguar el tipo 
        #de excepción adecuado
        if expresion:
            import string
            listavars = []
            #Copia de la lista, ya que vamos a hacer un reverse
            listavars += self._portero.actual().variables 
            #Evaluamos las variables con el nombre más grande para 
            #evitar los casos en los que una variable contenga a otra
            listavars.sort(lambda x, y: len(x.nombre) > len(y.nombre))
            listavars.reverse() 
            for var in listavars:
                numero = self._portero.actual().variables.obtenerindice(var)
                expresion = expresion.replace\
                        (var.nombre,"self.__getitem__(i)["+repr(numero)+"]")

            lista = self.__gestorpaquetes.listafunciones()
            cosatmp = self.__gestorpaquetes
            #Recorremos la lista de funciones, y hacemos la sustitucion
            for funcion in lista: 
                texto = string.split(funcion,".")
                ristra = "cosatmp[\""+texto[0]+"\"][\""+texto[1]+"\"]"
                expresion = expresion.replace(funcion, ristra)
            for i in range(self.numReg()):
                resultado = eval(expresion)
                self._portero.actual().registros[i].append(resultado) 
        else: #No nos pasan expresion, rellenamos con los valores por defecto
            for i in range(self.numReg()):
                self._portero.actual().registros()[i].append(valorpordefecto)

    def arrTitDiscreto(self):
        """Devuelve una lista con los titulos de las variables discretas"""
        lista=[]
        from Driza.variables import AgrupadorDiscreto
        for variable in self._portero.actual().variables():
            if isinstance(variable,AgrupadorDiscreto):
                lista.append(variable.nombre)
        return lista

    def arrTitNumerica(self):
        """Devuelve una lista con los titulos de las variables discretas"""
        lista=[]
        from Driza.variables import AgrupadorNumerico
        for variable in self._portero.actual().variables():
            if isinstance(variable,AgrupadorNumerico):
                lista.append(variable.nombre)
        return lista

    def arrTit(self, tipo = None, factores = True):
        """
        Devuelve una lista con todos los nombres de las variables
        opcionalmente podemos filtrar segun un tipo
        """
        if not tipo:
            if factores:
                return [variable.nombre for variable in \
                        self._portero.actual().variables()]
            else:
                lista=[]
                for variable in self._portero.actual().variables():
                    if variable.tipo!="Factor":
                        lista.append(variable.nombre)
                return lista

        else:
            lista=[]
            for variable in self._portero.actual().variables():
                if variable.tipo==tipo:
                    lista.append(variable.nombre)
            return lista
                
    def getCasos(self, indice = None):
        """Devuelve todos los casos existentes de una determinada variable"""
        #debe ser una variable discreta, si no excepcion
        lista=[]
        indiceentero = self._portero.actual().variables().obtenerindice(indice)
        for valor in self._portero.actual().registros():
            if valor[indiceentero] not in lista:
                lista.append(valor[indiceentero])
        lista.sort()
        return lista

    def getCol(self, indice, filtrado = True,  condiciones=[]):
        """Devuelve un array con todos los valores asociados a 
        la variable representada por indice 
        los parametros filtrado y caso son los referidos al filtro general 
        y a la variable de agrupacion general.
        Para uso de las clases del programa
        """
        lista=[]
        if not filtrado or not self.__gestorfiltro.variable:
            #No Hay filtrado
            for valor in self._portero.actual().registros():
                esvalido = True
                if not valor[self._portero.actual().\
                        variables().obtenerindice(indice)].valido():
                    esvalido = False
                for condicion in condiciones:
                    if not condicion(valor): 
                        #Tiene que verificar todas las condiciones
                        esvalido = False 
                if esvalido:
                    lista.append(valor[self._portero.actual()\
                            .variables().obtenerindice(indice)])
        else:
            #Hay filtrado
            for valor in self._portero.actual().registros():
                esvalido = True
                if not valor[self._portero.actual().variables\
                        .obtenerindice(indice)]:
                    esvalido = False
                for condicion in condiciones:
                    if not condicion(valor): 
                        #Tiene que verificar todas las condiciones
                        esvalido = False 
                if not valor[self._portero.actual().variables.obtenerindice(self.__gestorfiltro.variable)].valido():
                    esvalido = False #Esta filtrado
                if esvalido:
                    lista.append(valor[self._portero.actual().variables.obtenerindice(indice)])

        return lista

    def insertarestado(self, estado, flagoriginal = False):
        """Inserta un estado tal cual"""
        self._portero.insertarestado(estado, flagoriginal = flagoriginal)

    def original(self):
        """Devuelve True o False en funcion de si el estado actual es original"""
        return self._portero.actualoriginal()

    def setFlagoriginal(self):
        """Establece la flag original"""
        self._portero.setactualoriginal()

    def delVar(self, variable):
        """Borra una variable y su contenido asociado"""
        indice = self._portero.actual().variables.obtenerindice(variable)
        for reg in self._portero.actual().registros:
            del reg[indice]
        del self._portero.actual().variables[indice] 

class InterfazDatosUsuario(InterfazDatos):
    """La interfaz de datos con el usuario"""
    def modVar(self, variable, indice, valor):
        """
        Modifica una variable existente. 
        Es necesario para verificar que el nombre no esta ya ocupado
        Variable es el objeto variable, no el indice
        """
        if indice==0 or indice=="nombre":
            if valor in self.arrTit(factores = True):
                from  Driza.excepciones import VariableExisteException
                raise VariableExisteException()
        elif indice==2 or indice=="valorpordefecto":
            variable.valorpordefecto=valor
        elif indice==3 or indice=="descripcion":
            variable.descripcion=valor

    def setVar(self, indice, variable, columna = None):
        """
        Sobreescribe la variable asociada al indice.
        Hace una conversión de tipos según la representación interna de estos
        Si se le pasa la columna, entonces sobreescribira todos 
        los registros asociados a la variable
        """
        #Debe comprobar la correctitud del array y lanzar 
        #una excepcion si no vale
        array = []
        if variable.nombre in self.arrTit(factores = True) \
                and self.getVar(indice).nombre != variable.nombre:
            #Comprobamos que no se trata de un cambio de atributo 
            #de la variable
            from  Driza.excepciones import VariableExisteException
            raise VariableExisteException()
        if not columna:
            #Conversion directa (Inestable)
            for elemento in self.getCol(indice):
                array.append(repr(elemento))
            self._portero.actual().variables[indice] = variable
            for registro in self._portero.actual().registros:
                registro[indice] = variable.nuevoitem(array[indice])

        else:
            #Nos pasan la columna con los datos
            self._portero.actual().variables()[indice] = variable
            for registro in self._portero.actual().registros():
                registro[indice] = variable.nuevoitem(columna[indice])
        #TODO: Continuar con el caso de la columna


class InterfazDatosR(InterfazDatos):
    """La interfaz de datos con R"""
    def RData(self):
        """Devuelve una copia de los datos actuales en el formato data de R"""
        #TODO: Falta el control de filtrado
        import rpy
        diccionario={}
        for variable in self._portero.actual().variables:
            if variable.tipo!="Factor":
                lista = map(float, self.__getitem__(variable))
            else:
                lista = map(str, self.__getitem__(variable))
            diccionario[variable.nombre]=lista
        df = rpy.r.data_frame(diccionario)
        return df

    def query(self, variable, *condiciones):
        """El query es una peticion de datos que sera enviada a R. 
        Hace prácticamente lo mismo que getCol, salvo que en este 
        caso la lista de datos es devuelta directamente en formato float, 
        y acepta condiciones de filtrado
        Para uso en las operaciones
        """
        #TODO assert la variable de filtrado es distinta de la 
        #de alguna condicion
        #TODO assert la variable es distinta a la 
        #de alguna condicion
        from Driza.condiciones import parsecondicion
        mifuncion = lambda f: parsecondicion(self, f)
        listacondiciones = map(mifuncion, condiciones)
        lista = self.getCol(variable, True, listacondiciones)
        lista = [f for f in lista if f.valido()]
        if self.getVar(variable).tipo!="Factor":
            return [float(x) for x in lista]
        else:
            return [ x for x in lista]

class InterfazDatosFicheros(InterfazDatos):
    """La interfaz de datos con los ficheros"""
    def loadTxt(self,fichero,**opciones):
        import os
        if not os.path.exists(fichero):
            from Driza.excepciones import FicheroNoExisteException
            raise FicheroNoExisteException(fichero)
        cabeceras=opciones["cabeceras"]
        delimitadoratrib=opciones["delimitadoratrib"]
        from rpy import r
        datos = r.read_table(fichero, sep = delimitadoratrib, \
                header = cabeceras, na_strings="NA", dec=".", \
                strip_white = False)
        for pareja in datos.items():
            import types
            milista=[]
            if type(pareja[1][0])==types.IntType:
                self.appVar(pareja[0],"Entero")
                for x in pareja[1]:
                    if x==-2147483648:
                        resultado=float('nan')
                    else: resultado = x
                    milista.append(resultado)
            elif type(pareja[1][0])==types.StringType:
                #Es un Factor
                self.appVar(pareja[0],"Factor")
                milista=pareja[1]
            elif type(pareja[1][0])==types.FloatType:
                self.appVar(pareja[0],"Real")
                milista=pareja[1]
            else: print "ERROR",pareja[1][0] #TODO: Cambiar por excepcion

            self.__setitem__(self.getVar(pareja[0]),milista)

