#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Módulo de gestores"""

from Driza.clasesbase import OperacionCalculo
from Driza.clasesbase import OperacionDistribucion
from Driza.clasesbase import OperacionGraficos
from Driza.clasesbase import OperacionMultiple
from Driza.clasesbase import OperacionTabla



class GestorOperaciones(dict):
    """
    Gestiona las operaciones que están activas en la ejecución 
    del programa
    """
    def __init__(self, dato, config):
        dict.__init__(self)
        self.__dato = dato
        self.__config = config
        self.__initintegrado()

    def __initintegrado(self):
        """Esta funcion carga en el diccionario las operaciones"""
        modules=self.__listamodulos()
        for operacion in modules:
            #try:
            if 1:
                tipo = operacion.tipo
                widget = self.__cargawidgetdedescripcion(operacion.nombre,operacion.widget)
                definicionresultado = self.__cargardefiniciondedescripcion(operacion.nombre,operacion.definicionresultado)
                if tipo == "Calculo":
                    mioperacion = OperacionCalculo(operacion.nombre, \
                            self.__dato, self.__config, operacion.descripcion, widget, \
                            definicionresultado,operacion.funcionprincipal, \
                            operacion.funcionchequeocondiciones, \
                            operacion.funcionchequeoentradausuario)    
                elif tipo == "Distribucion":
                    mioperacion = OperacionDistribucion(operacion.nombre, \
                            self.__dato, self.__config, widget, operacion.funcion, \
                            operacion.interfazresultado, \
                            operacion.initresultado,operacion.comprobarentrada)    
                elif tipo == "Graficos":
                    mioperacion = OperacionGraficos(operacion.nombre, \
                            self.__dato, self.__config, widget, operacion.funcion, \
                            operacion.interfazresultado, \
                            operacion.initresultado,operacion.comprobarentrada)    
                elif tipo == "Tabla":
                    mioperacion = OperacionTabla(operacion.nombre, \
                            self.__dato, self.__config, widget, operacion.funcion, \
                            operacion.interfazresultado, \
                            operacion.initresultado,operacion.comprobarentrada)    
                elif tipo == "Multiple":
                    mioperacion = OperacionMultiple(operacion.nombre, \
                            self.__dato, self.__config, widget, operacion.funcion, \
                            operacion.interfazresultado, \
                            operacion.initresultado,operacion.comprobarentrada)    
                self.__setitem__(operacion.nombre, mioperacion)

            #except: #TODO: Poner el tipo de excepcion
            #    print u"Módulo erróneo:"+unicode(operacion.nombre)

    def __listamodulos(self):
        """Devuelve una lista con los modulos ya cargados"""
        #Sacado de:
        # http://www.diveintopython.org/functional_programming/all_together.html
        import Driza.operaciones
        ruta = Driza.operaciones.__path__[0]
        import os
        import re
        ficheros = os.listdir(ruta)
        del ficheros[ficheros.index("__init__.py")]
        test = re.compile(".*\.py$", re.IGNORECASE)
        #ficheros = filter(test.search, ficheros)   
        ficheros = ["compmeddob.py"]
        filenametomodulename = lambda f: os.path.splitext(f)[0]
        modulenames = map(filenametomodulename, ficheros)
        mifuncion = lambda f: "Driza.operaciones."+f
        modulenames = map(mifuncion, modulenames)
        mifuncion2 = lambda f: __import__(f, None, None, ["Driza.operaciones"])
        modules = map(mifuncion2, modulenames) 
        return modules

    def __cargawidgetdedescripcion(self,nombre,diccionariowidget):
        """Carga el widget a partir del diccionario que figura en el modulo de la operacion"""
        from Driza.widgets import woperaciones
        if diccionariowidget["tipo"]=="CalculoSimple":
            widget=woperaciones.WidgetCalculoSimple(nombre,diccionariowidget["opciones"],datos)
        elif diccionariowidget["tipo"]=="BondadDeAjuste":
            widget=woperaciones.WidgetCalculoSimpleBondadAjuste(nombre,diccionariowidget["listaoperaciones"],datos)
        elif diccionariowidget["tipo"]=="CalculoDoble":
            widget=woperaciones.WidgetCalculoDoble(nombre,diccionariowidget["opciones"],self.__dato)
        elif diccionariowidget["tipo"]=="ProporcionSimple":
            widget=woperaciones.WidgetCaso(nombre,diccionariowidget["opciones"],datos)
        elif diccionariowidget["tipo"]=="Graficos":
            widget=woperaciones.WidgetGraficos(nombre,diccionariowidget["graficos"],diccionariowidget["opciones"],datos)
        elif diccionariowidget["tipo"]=="Distribucion":
            widget=woperaciones.WidgetDistribucion(nombre,diccionariowidget["opcionescomun"],[],self.__dato)
        elif diccionariowidget["tipo"]=="2Variables3Casos":
            widget=woperaciones.Widget2Variable3Casos(nombre,diccionariowidget["opciones"],self.__dato)
        elif diccionariowidget["tipo"]=="VariableCasoCaso":
            widget=woperaciones.WidgetVariableCasoCaso(nombre,diccionariowidget["opciones"],self.__dato)
        elif diccionariowidget["tipo"]=="VariableCasoCasoVariableVariable":
            widget=woperaciones.WidgetVariableCasoCasoVariableVariable(nombre,diccionariowidget["opciones"],self.__dato)
        elif diccionariowidget["tipo"]=="VariableCasoCasoVariable":
            widget=woperaciones.WidgetVariableCasoCasoVariable(nombre,diccionariowidget["opciones"],self.__dato)
        else:
            raise NameError
        return widget

    def __cargardefiniciondedescripcion(self,nombre,listadiccionariodefinicion):
        """Carga la definicion de los resultados a partir del diccionario suministrado por el modulo de la operacion"""
        from Driza.salida import FormatoResultado
        fresultado=FormatoResultado(nombre,self.__config)
        for diccionariodefinicion in listadiccionariodefinicion:
            elemento = None
            if diccionariodefinicion["tipo"]=="Texto":
                from Driza.salida import DefinicionElementoResultadoParrafo
                elemento=DefinicionElementoResultadoParrafo(diccionariodefinicion) #Que coja el los campos del diccionario
            elif diccionariodefinicion["tipo"]=="Tabla":
                from Driza.salida import DefinicionElementoResultadoTabla
                elemento=DefinicionElementoResultadoTabla(diccionariodefinicion["nombre"],diccionariodefinicion) #Que coja el los campos del diccionario
            elif diccionariodefinicion["tipo"]=="Fila":
                from Driza.salida import DefinicionElementoResultadoFila
                elemento=DefinicionElementoResultadoFila(diccionariodefinicion["nombre"],diccionariodefinicion) #Que coja el los campos del diccionario
            elif diccionariodefinicion["tipo"]=="Imagen":
                from Driza.salida import DefinicionElementoResultadoImagen
                elemento=DefinicionElementoResultadoFila(diccionariodefinicion) #Que coja el los campos del diccionario
            fresultado[diccionariodefinicion["nombre"]]=elemento
        return fresultado

    def funcionchequeocondiciones(self,nombre):
        """Realiza el chequeo de condiciones llamando a la funcion suministrada por la operacion, pasando como parametros la interfazdedatos"""
        return self.__getitem__(nombre).funcionchequeocondiciones(self.__dato)

class Puntero:
    """
    Clase que sirve a otras clases para acceder a la clase 
    que actualmente esta desempeñando una funcion
    """
    def __init__(self):
        self.variable = None

    def set(self, variable):
        """Establece el valor de la variable"""
        self.variable = variable
    def unset(self):
        """Elimina el valor de la variable"""
        self.variable = None


class PunteroFiltro(Puntero):
    """Almacena la variable que actua como filtro"""
    def set(self, filtro):
        """Establece el valor del filtro"""
        #from Driza.variables import TBooleano
        #assert isinstance(filtro, TBooleano)
        #Puntero.set(self, filtro)
        pass
        #TODO: Rehacer
    

class GestorConfig:
    """
    Gestor de configuracion. Se encarga de controlar si esta cargada la configuracion
    si el formato es el más actual, y mantiene una referencia al propio objeto
    de configuración
    """
    def __init__(self):
        import os
        self.__fichero = os.environ["HOME"]+"/.driza"
        self.configuracion = None

    def load(self, fichero = None):
        """Carga un fichero de configuracion, por defecto ~/.driza"""
        from Driza.interfaz import InterfazConfig
        from Driza.clasesbase import Configuracion
        from Driza.excepciones import VersionAnterior
        nuevo = False
        if fichero:
            self.__fichero = fichero
        ifc = InterfazConfig()
        configuracion = None
        try:
            configuracion = ifc.load(self.__fichero)
        except IOError:
            nuevo = True
            configuracion = Configuracion()
            ifc.save(configuracion, self.__fichero)
        except VersionAnterior(configuracion):
            #Aqui van las correcciones para cada revision
            if configuracion["version"] < 2:
                del configuracion["primeravez"]
                configuracion["nundo"] = 5 #TODO: Ver si esto hace falta (al menos es ilustrativo)
                configuracion["decimales"] = 3
            configuracion["version"] = 2

            self.configuracion=configuracion
        except:
            print "Error desconocido!, Intente borrar el fichero '~/.driza'"
        
        self.configuracion = configuracion
        return nuevo #Devuelve si el fichero es nuevo o no
            
    def save(self, fichero = None):
        """Guarda el fichero de configuracion en un fichero"""
        from Driza.interfaz import InterfazConfig
        if fichero:
            self.__fichero = fichero
        ifc = InterfazConfig()
        ifc.save(self.configuracion, self.__fichero)




class GestorProyectos:
    """
    Almacena la variable que actua como fichero.  
    Dispone además de funciones de carga y almacenamiento
    """
    def __init__(self, portero, config):
        self.fichero = None
        self.__config = config
        self.__portero = portero
        import re
        self.__driza = re.compile('.*\.driza$')
        self.__txt = re.compile('.*\.txt$')


    def guardar(self, fichero=None, sobreescribir=False):
        """Guarda la copia actual de los datos en el fichero indicado"""
        tipo = "Auto"
        if fichero: 
            self.fichero = fichero
        if not self.fichero:
            from Driza.excepciones import NombreFicheroNoIndicadoException
            raise NombreFicheroNoIndicadoException
        import re
        extension = re.compile('.*\.driza$')
        if (tipo == "Auto" and self.__driza.match(self.fichero)) or tipo == "Driza":
            self.__saveDriza(self.fichero, self.__portero.actual(), sobreescribir)
            self.__config.configuracion["lfichero"].insert(self.fichero)
        else:
            from Driza.excepciones import FicheroTipoDesconocidoException
            raise FicheroTipoDesconocidoException(self.fichero)

    def cargar(self, fichero, tipo = "Auto", opciones = None):
        """Carga los datos del fichero indicado"""
        self.fichero = fichero
        self.__portero.guardarestado()
        if (tipo == "Auto" and self.__driza.match(fichero)) or tipo == "Driza":
            datos = self.__loadDriza(self.fichero)
       # elif (tipo =="Auto" and self.__txt.match(fichero)) or tipo == "Texto":
       #     self.__id.borrartodo()
       #     datos = self.__if.loadTxt(self.fichero,self.__id,opciones["delimitadoratrib"],opciones["cabeceras"])
        else:
            from Driza.excepciones import FicheroTipoDesconocidoException
            raise FicheroTipoDesconocidoException(fichero)
        self.__portero.insertarestado(datos, flagoriginal=True)
        self.__config.configuracion["lfichero"].insert(self.fichero)

#    def importartexto(self, fichero, delimitadoratrib = "\t", \
#            cabeceras = False):
#        """Importa un fichero de texto"""
#        self.__if.loadTxt(fichero, {"delimitadoratrib":delimitadoratrib, "cabeceras":cabeceras})
        
    def __saveDriza(self, fichero, datos, sobreescribir = False):
        """
        Guarda un fichero en el formato standard (Pickle) 
        """
        import os
        if not sobreescribir and os.path.exists(fichero): #El fichero existe
            from Driza.excepciones import FicheroExisteException
            raise FicheroExisteException(fichero)
        archivo = open(fichero, 'w')
        import pickle
        pickle.dump(datos, archivo)
        archivo.close()

    def __loadDriza(self, fichero):
        """
        Carga un fichero tipo Driza
        """
        import os
        if not os.path.exists(fichero):
            from Driza.excepciones import FicheroNoExisteException
            raise FicheroNoExisteException(fichero)
        archivo = open(fichero, 'r')
        import pickle
        datos = pickle.load(archivo) 
        archivo.close()
        return datos


class GestorTemas:
    """
    Gestiona los temas. Su utilidad real queda pendiente 
    de futuros diseños
    """
    def __init__(self):
        import os
        self.temaactual = None
        self.ruta = None
        posiblesrutas = ["Driza/images/", \
                "/usr/lib/python2.4/site-packages/Driza/images/", \
                "~/.driza/images/"]
        for ruta in posiblesrutas:
            if os.path.exists(ruta):
                self.ruta = ruta
                break

        if not self.ruta:
            print "ERROR" #TODO: Raise excepción

    def portada(self):
        """Devuelve el QPixmap con la imagen de portada"""
        from qt import QPixmap
        return QPixmap(self.ruta + "driza.png")

    def iconoabrir(self):
        """Devuelve un QPixmap con el icono de abrir"""
        from qt import QPixmap
        return QPixmap(self.ruta + "botonabrir.png")

    def iconoguardar(self):
        """Devuelve un QPixmap con el icono de guardar"""
        from qt import QPixmap
        return QPixmap(self.ruta + "botonguardar.png")

    def iconoexportar(self):
        """Devuelve un QPixmap con el icono de exportar"""
        from qt import QPixmap
        return QPixmap(self.ruta + "botonexportar.png")


class GestorPaquetes(dict):
    """
    Gestiona los paquetes que contendran las funciones. Estos paquetes son 
    equivalentes al modulo de python que los contiene
    """
    def __init__(self):
        dict.__init__(self)
        basicas = Paquete("basicas")
        self.__setitem__("basicas", basicas)
        #Intenta cargar el modulo
        #Cargaria todas las funciones de este
        #Leeria una descripción del modulo 
        #(Atributo "descripcion" dentro del ficher)
        #Cada funcion tendria un parametro nombre, y otro descripcion

        #La estructura final seria:
        #{PAritmeticas:[FSuma,FResta,etc...],...} #PAritmeticas es un paquete
        # El dialogo o lo que sea consultaria a cada funcion el nombre 
        # y la descripcion

    def listafunciones(self):
        """Devuelve una lista con todos los paquetes y todas las funciones"""
        lista = []
        for paquete in self.items():
            for funcion in paquete[1].keys():
                lista.append(paquete[1].nombrereal + "." + funcion)
        return lista


class Paquete(dict):
    """Representa un paquete. Un paquete es un conjunto de funciones"""
    def __init__(self, nombre):
        """Carga el paquete de una ruta"""
        dict.__init__(self)
        ruta = "Driza.funciones." + nombre
        self.nombrereal = nombre # El nombre real del paquete
        mifuncion2 = lambda f: __import__(f, None, None, ["Driza.funciones"])
        modulo = mifuncion2(ruta)
        listamiembros = dir(modulo)
        import re
        principal = re.compile('^[A-Z]+')
        nuevalista = filter(principal.match, listamiembros)
        del nuevalista[nuevalista.index("Funcion")]
        nuevalista2 = []
        for nombremodulo in nuevalista:
            nombre = "modulo" + "." + nombremodulo + "()"
            nuevalista2.append(eval(nombre))
        for indicepaquete in range(len(nuevalista)):
            self.__setitem__(nuevalista[indicepaquete], \
                    nuevalista2[indicepaquete]) 
            #Ponemos valor al diccionario

class GestorMensajesCategorias(dict):
    """Este gestor guarda un mensaje asociado a cada categoria.
    Se utiliza los operadores de diccionario para [] acceso y modificación
    Devuelve un QLabel con el texto"""
    def __init__(self):
        dict.__init__(self)

    def __setitem__(self, indice, parametro):
        if isinstance(parametro,str) or isinstance(parametro,unicode):
            from qt import QLabel
            label=QLabel(None)
            label.setText(parametro)
            dict.__setitem__(self,indice,label)
       
