#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Fichero manejador de ficheros
#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Módulo de interfaces"""

from rpy import r
from ConfigParser import ConfigParser

class InterfazConfig:
    """
    Clase encargada de actuar como interfaz de la configuración. Permite 
    leer y guardar la configuración en ficheros.
    """
    def load(self, fichero):
        """
        Carga la configuración del fichero de configuración del programa, 
        devolviendo la configuración por defecto si no la encuentra. 
        El parametro fichero es opcional e indica un fichero alternativo 
        al de configuracion por defecto 
        """
        configfile = ConfigParser()
        from Driza.clasesbase import Configuracion
        config = Configuracion()
        archivo = open(fichero,"r")
        configfile.readfp(archivo)
        config.creaDesdeTexto(configfile._sections["General"])
        if config["version"]<2:
            from Driza.excepciones import VersionAnterior
            raise VersionAnterior(configuracion)
        archivo.close()
        return config


    def save(self, config, fichero):
        """
        Guarda la configuracion en el fichero de configuración del programa. 
        El parametro fichero es opcional e indica un fichero alternativo al 
        de configuracion por defecto
        """
        configfile = ConfigParser()
        config["lfichero"] = ",".join(config["lfichero"])
        configfile.add_section("General")

        for oper, valor in config.items():
            configfile.set("General", oper, valor)

        archivo = open(fichero, "w")
        configfile.write(archivo)
        archivo.close()

        #Volvemos a poner a un tipo ListaFicheros
        from Driza.clasesbase import ListaFicheros
        config["lfichero"] = ListaFicheros(5, config["lfichero"].split(","))

