#!/usr/bin/python
# -*- coding: utf-8 -*-
#Clases de elementos correspondientes a la ventana de salida

#salida.py: Objetos relacionados con la ventana de salida

#Copyright (C) 2006-2007  Inmaculada Luengo Merino, Nstor Arocha Rodrguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by

#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
class FormatoResultado(dict):
    """Almacena las diferentes definiciones de resultado"""
    def __init__(self,nombre,config):
        self.__nombre=nombre
        self.__config=config
        dict.__init__(self)
        #FALTA  un nombre para genrar titulo a la salida
    def __setitem__(self,indice,elemento):
        """Equivalente al append de list, salvo que controla los tipos"""
        assert(isinstance(elemento,DefinicionElementoResultado))
        dict.__setitem__(self,indice,elemento)
    def renderizar(self,listaresultados):
        """Devuelve un ResultadoOperacion con todos sus elementos ya rellenado"""
        resultadooperacion=ResultadoOperacion()
        titulo=TextoResultado()
        titulo.set(self.__nombre)
        resultadooperacion["titulo"]=titulo
        listafiltrada=[]
        for (indice,objetodefinicion) in self.items():
            for diccionario in listaresultados:
                listafiltrada.append(diccionario[indice]) #Solo enviamos los diccionarios destinados a ese elemento de salida
            resultadooperacion[indice]=objetodefinicion.renderizar(listafiltrada)
        return resultadooperacion


class ResultadoOperacion(dict):
    """Esta clase representa un resultado, que es un conjunto de DefinicionElementosresultado. 
    Posee funciones para conversion a html
    """
    def __init__(self):
        self.titulo=""
        dict.__init__(self)


    def html(self):
        """Genera el html a partir de todos sus componentes + el titulo"""
        ristra=u""
        ristra+="<h1>"+self.titulo+"</h1>"
        for elemento in self.values():
            ristra+=elemento.html()
        return ristra
        
    def exportarhtml(self,ruta):
        """Genera el html a partir de todos sus componentes + el titulo"""
        ristra=u""
        ristra+="<h1>"+self.titulo+"</h1>"
        for elemento in self.values():
            ristra+=elemento.exportarhtml(ruta)
        return ristra

class DefinicionElementoResultado:
    """Esta clase define un elemento del resultado.
    En la inicializacin se definen los elementos estticos, como los titulos y bordes
    Adems, se ofrece una clase para agregar elementos que es empleada por las clases de Operacion
    """
    def __init__(self,nombre):
        self.nombre=nombre

class DefinicionElementoResultadoTabla(DefinicionElementoResultado):
    """Define una tabla por cada resultado"""
    def __init__(self,nombre,diccionarioopciones):
        DefinicionElementoResultado.__init__(self,nombre)
        if diccionarioopciones.has_key("borde"):
            pass
        self.cabecera=diccionarioopciones["cabecera"]

    def renderizar(self,listadiccionarios):
        """Aade una nueva tabla. El parametro ha de ser una lista (filas)con listas (columnas)"""
        resultado=ListaResultado()
        titulo=TextoResultado()
        titulo.set(self.nombre)
        resultado.append(titulo)
        for diccionario in listadiccionarios:
            listatmp=[]
            for indice in self.cabecera:
                listatmp.append(diccionario[indice])
            tabla=TablaResultado()
            tabla.append(self.cabecera)
            tabla.append(listatmp)
            resultado.append(tabla)
        return resultado


class DefinicionElementoResultadoFila(DefinicionElementoResultado):
    """Define una tabla a la que cada llamada le aade una fila"""
    def __init__(self,nombre,diccionarioopciones):
        DefinicionElementoResultado.__init__(self,nombre)
        if diccionarioopciones.has_key("borde"):
            pass
        self.cabecera=diccionarioopciones["cabecera"]

    def renderizar(self,listadiccionarios):
        """Aade una nueva tabla. El parametro ha de ser una lista (filas)con listas (columnas)"""
        resultado=ListaResultado()
        titulo=TextoResultado()
        titulo.set(self.nombre)
        resultado.append(titulo)
        tabla=TablaResultado()
        tabla.append(self.cabecera)
        for diccionario in listadiccionarios:
            listatmp=[]
            for indice in self.cabecera:
                listatmp.append(diccionario[indice])
            tabla.append(listatmp)
        resultado.append(tabla)
        return resultado

class DefinicionElementoResultadoImagen(DefinicionElementoResultado):
    pass

class DefinicionElementoResultadoParrafo(DefinicionElementoResultado):
    def __init__(self,nombre,diccionario = None):
        """El diccionario es leido del fichero de la operacion"""
        DefinicionElementoResultado.__init__(self,nombre)
        if diccionario:
            self.__tamanofuente=diccionario["tamanofuente"]

    def renderizar(self,diccionario):
        """El diccionario es recibido por la funcionprincipal de la operacion"""
        miresultado=TextoResultado()
        miresultado.set(diccionario["contenido"])
        return miresultado


class ElementoResultado:
    """Cualquier elemento que figure en un resultado ser hijo de esta clase"""
    def __init__(self):
        pass

    def set(self,dict):
        """Abstracta: define como ha de recibir cada elemento la informacin del diccionario dict"""
        pass

    def html(self):
        """Abstracta: muestra el contenido del widget volcado a html"""
        return ""

    def exportarhtml(self,fichero):
        """Esta funcion es llamada cuando se desea exportar html a un fichero. Normalmente hace lo mismo que html()"""
        return self.html()


class Tabla:
    """Almacena una tabla. Esta clase asegura que se cumple la restriccion del tamao para todos los registros"""
    def __init__(self):
        self.__contenido=list()
        self.__ncols=None
        self.__listatitulos=None

    def settitulo(self,args):
        self.__listatitulos=args

    def append(self,args):
        """Introduce los argumentos en la tabla. Ha de coincidir en tamao con ncols.Recibe ristras"""
        if not self.__ncols:
            self.__ncols=len(args)
        if len(args) != self.__ncols:
            print "error"
        else:
            self.__contenido.append(args)


    def html(self):
        """Devuelve el contenido de la tabla en html"""
        cadena=""
        if self.__listatitulos:
            cadena+="<tr>\n"
            for titulo in self.__listatitulos:
                cadena+="<td>"+titulo+"</td>"
            cadena+="</tr>\n"

        for x in self.__contenido:
            cadena+="<tr>\n"
            for y in x:
                if isinstance(y,list) or isinstance(y,float) or isinstance(y,dict):
                    y=str(y)
                cadena+="<td>"+y+"</td>\n"
            cadena+="</tr>\n"
        return cadena

class ListaResultado(ElementoResultado,list):
    """Varios resultados agrupados"""
    def html(self):
        ristra=""
        for x in self:
            ristra+=x.html()
        return ristra

class TablaResultado(ElementoResultado):
    """Clase antesesora de todas las tablasresultados"""
    def __init__(self):
        ElementoResultado.__init__(self)
        self.contenido=Tabla()
        self.titulo=""

    def html(self):
        ristra=""
        ristra+="<h2>"+self.titulo+"</h2>"
        ristra+="<table border=\"1\">"
        ristra+=self.contenido.html()
        ristra+="</table>"
        return ristra

    def append(self,listaargs):
        """Debe recibir ristras"""
        self.contenido.append(listaargs)

    def settitulo(self,titulo):
        self.contenido.settitulo(titulo)



class TablaResultadoSimple(TablaResultado):
    """Una tabla. Convierte un array de arrays en una tabla html"""
    def set(self,variable,listaargs):
        """Carga los argumentos en la tabla, pero solo los que no sean nulos"""
        self.contenido.set([variable]+listaargs)

    def settitulo(self,titulo):
        """Estable el titulo de la tabla"""
        self.contenido.settitulo(["Variable"]+titulo)


class TablaResultadoDoble(TablaResultado):
    """Una tabla. Convierte un array de arrays en una tabla html"""
    def set(self,variable,listaargs):
        """Carga los argumentos en la tabla, pero solo los que no sean nulos"""
        self.contenido.set([variable[0],variable[1]]+listaargs)

    def settitulo(self,titulo):
        """Estable el titulo de la tabla"""
        self.contenido.settitulo(["Variable1","Variable2"]+titulo)

class TextoResultado(ElementoResultado):
    """Es un texto cualquiera"""
    def __init__(self):
        ElementoResultado.__init__(self)
        self.contenido=""
        self.titulo=""

    def set(self,texto):
        if isinstance(texto,str):
            self.contenido=texto
        elif isinstance(texto,unicode):
            self.contenido=texto.encode('iso-8859-1','replace')
        else:
            self.contenido(str(texto))

    def html(self):
        if self.contenido:
            ristra= self.contenido+"<br>"
            return unicode(ristra,'iso-8859-1')
        else: 
            return u""

class ImagenResultado(ElementoResultado):
    """Representa una imagen"""
    def __init__(self):
        ElementoResultado.__init__(self)
        self.contenido=None #TODO
        self.ruta=None

    def set(self,ruta):
        self.ruta=ruta

    def settitulo(self,titulo):
        self.titulo=titulo

    def exportarhtml(self,carpeta):
        """Copia el archivo a la carpeta donde estaran las imagenes, y devuelve el html con la ruta de la imagen actualizada
        La carpeta ya debe estar creada
        """
        import os
        import shutil
        ristra=""
        if self.ruta:
            (dirName, fileName) = os.path.split(self.ruta)
            shutil.copyfile(self.ruta,carpeta+"/"+fileName)
            ristra+="<h2>"+self.titulo+"</h2>"
            ristra+="<img src=\""+os.path.split(carpeta)[1]+"/"+fileName+"\">"
        return ristra

    def html(self):
        ristra=""
        if self.ruta:
            ristra+="<h2>"+self.titulo+"</h2>"
            ristra+="<img src=\""+self.ruta+"\">"
        return ristra


