#!/usr/bin/python
# -*- coding: utf-8 -*-
#####
# Fichero de variables
# define las clases de variables y agrupadores de variables
####

#Copyright (C) 2006-2007  Inmaculada Luengo Merino, Nstor Arocha Rodrguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

class Variable:
    """La clase padre de todas las variables"""
    def __init__(self,agrupador,valor):
        self.tvariable=agrupador
        self.valor=self._interno(valor)

    def valido(self):
        if self.valor==None: 
            return False
        return True

class VariableNumerica:
    """Las clases que heredan son numericas. Esta clase solo aporta esa informacion de momento"""

class VariableDiscreta(Variable):
    """Clase que representa a las variables discretas. Permiten la comparacion directa entre valores"""
    def __eq__(self,other):
        return self.valor==other.valor

class VariableContinua(Variable):
    """Representa a las variables continuas"""
    def __repr__(self):
        pass
    def __float__(self):
        return 0

class Factor(VariableDiscreta):
    """Las variables que solo son cadenas y que nicamente sirven para agrupacin"""
    def _interno(self,cadena):
        if cadena=="''":
            return str()
        if isinstance(cadena,str):
            return cadena
        else:
            return str(cadena)

    def __str__(self):
        if self.valido():
            return self.valor
        return ''

    def __repr__(self):
        if self.valido():
            return self.valor
        return ''

class Real(VariableContinua,VariableNumerica):
    def _interno(self,valor):
        if valor=="None" or valor=="NA" or valor=="nan" or valor==float("nan"):
            return None
        else:
            return float(valor)
    def __float__(self):
        return self.valor
    def __repr__(self):
        return repr(self.valor)
    def __str__(self):
        return str(self.valor)


class Entero(VariableDiscreta,VariableNumerica):
    def _interno(self,valor):
        if valor=="None" or valor=="NA" or valor=="nan" or valor==float("nan"):
            return None
        else:
            return int(valor)
    def __float__(self):
        if self.valido():
            return float(self.valor)
        else:
            return float('nan')
    def __repr__(self):
        return repr(self.valor)


class Agrupador:
    """Clase de tipo que es padre de todos los Agrupadores.
    Un agrupador son aquellas caracteristicas de una variable de estudio
    """
    def __init__(self,nombre,descripcion=""):
        self.nombre=nombre
        self.descripcion=descripcion
        self.diccionarioconversion={}

    def update(self,objeto):
        """Funcin de copia de los atributos para cualquier Agrupador.
        Sirve para hacer copias entre Agrupadores
        """
        self.nombre=objeto.nombre
        self.descripcion=objeto.descripcion

class AgrupadorDiscreto:
    """Sirve para saber si la clase es discreta o no"""
    pass

class AgrupadorNumerico:
    """Sirve para saber si la clase es numerica o no"""
    pass

class AgrupadorFactor(Agrupador,AgrupadorDiscreto):
    """Clase de tipo que representa a los factores"""
    def __init__(self,nombre,descripcion=""):
        Agrupador.__init__(self,nombre,descripcion)
        self.tipo="Factor"
        self.diccionarioconversion={"Agrupador":["CDefault"]}
        self.valorpordefecto=""

    def nuevoitem(self,ristra):
        """Crea un nuevo factor cuyo padre es este agrupador"""
        return Factor(self,ristra)


class AgrupadorEntero(Agrupador,AgrupadorDiscreto,AgrupadorNumerico):
    """
    La clase entero. Almacena cualquier nmero entero,
    garantizando las operaciones de transformacin a formato interno y a ristra.
    """
    def __init__(self,nombre,valorpordefecto=0,descripcion=""):
        """
        Inicializacion de TVariable entera
        """
        Agrupador.__init__(self,nombre,descripcion)
        self.tipo="Entero"
        self.diccionarioconversion={"Agrupador":["CDefault"]}
        self.valorpordefecto=valorpordefecto
    
    def nuevoitem(self,valor):
        return Entero(self,valor)

class AgrupadorReal(Agrupador,AgrupadorNumerico):
    """
    La clase Real. Almacena cualquier nmero real, 
    garantizando las operaciones de transformacin a formato interno y a ristra. 
    """
    def __init__(self,nombre,valorpordefecto=0.0,descripcion=""):
        """Inicializacion del objeto Real"""
        Agrupador.__init__(self,nombre,descripcion)
        self.tipo="Real"
        self.valorpordefecto=valorpordefecto
        self.diccionarioconversion={"Agrupador":["CDefault"]}

    def nuevoitem(self,valor):
        return Real(self,valor)


